//---------------------------------------------------------------------------
#include <stdio.h>
#include "Allometry.h"

/*///////////////////////////////////////////////////////////////////////////
CalcAdultHeight()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////*/
float clAllometry::CalcAdultHeight( float fDbh, int iSpecies ) {
 try {
   float fHeight;
   if (iSpecies >= mp_oTreePop->m_iNumSpecies || iSpecies < 0) {
     //invalid species - throw an error
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::CalcAdultHeight");
     sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies);
     throw(stcErr);
   }
   fHeight = 1.35 + ((mp_oTreePop->mp_fMaxTreeHeight[iSpecies] - 1.35)*
          (1.0-exp((double)(-mp_oTreePop->mp_fSlopeAsympHeight[iSpecies] * fDbh))));

   return fHeight;
 }
 catch(modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clAllometry::CalcAdultHeight");
   throw(stcErr);
 }
}
//--------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
CalcAdultDbh()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////*/
float clAllometry::CalcAdultDbh( float fHeight, int iSpecies ) {
 try {
   float fDbh;
   if (iSpecies >= mp_oTreePop->m_iNumSpecies || iSpecies < 0) {
     //invalid species - throw an error
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::CalcAdultDbh");
     sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies);
     throw(stcErr);
   }
   //If the height is greater than or equal to canopy height, throw bad data error
   if (fHeight >= mp_oTreePop->mp_fMaxTreeHeight[iSpecies]) {
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::CalcAdultDbh");
     sprintf(stcErr.cMoreInfo, "%s%s", "Cannot set tree height greater than ",
         "maximum canopy height for this species.");
     throw(stcErr);
   }
   fDbh = log(1-((fHeight-1.35)/(mp_oTreePop->mp_fMaxTreeHeight[iSpecies]-1.35)))/
    (-mp_oTreePop->mp_fSlopeAsympHeight[iSpecies]);
   return fDbh;
 }
 catch (modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clAllometry::CalcAdultDbh");
   throw(stcErr);
 }
}
//--------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
CalcAdultCanRadius()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////*/
float clAllometry::CalcAdultCanRadius( float fDbh, int iSpecies ) {
 try {
   float fCanrad;
   if (iSpecies >= mp_oTreePop->m_iNumSpecies || iSpecies < 0) {
     //invalid species - throw an error
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::CalcAdultCanRadius");
     sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies);
     throw(stcErr);
   }
   fCanrad = mp_oTreePop->mp_fCanopyRadToDbh[iSpecies] *
              pow(fDbh, mp_oTreePop->mp_fCanopyRadExp[iSpecies]);
   if (fCanrad > MAX_CANOPY_RADIUS)
     fCanrad = MAX_CANOPY_RADIUS;
   return fCanrad;
 }
 catch(modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clAllometry::CalcAdultCanRadius");
   throw(stcErr);
 }
}
//--------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
CalcAdultCanDepth()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////*/
float clAllometry::CalcAdultCanDepth( float fHeight, int iSpecies ) {
 try {
   float fCanDepth;
   if (iSpecies >= mp_oTreePop->m_iNumSpecies || iSpecies < 0) {
     //invalid species - throw an error
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::CalcAdultCanDepth");
     sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies);
     throw(stcErr);
   }
   fCanDepth = mp_oTreePop->mp_fFractOfHeightInCan[iSpecies] *
              pow(fHeight, mp_oTreePop->mp_fCanopyDepthExp[iSpecies]);
   return fCanDepth;
 }
 catch(modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clAllometry::CalcAdultCanDepth");
   throw(stcErr);
 }
}
//--------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
CalcSeedlingHeight()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////*/
float clAllometry::CalcSeedlingHeight( float fDiam10, int iSpecies ) {
 try {
   float fHeight = 0;
   if (iSpecies >= mp_oTreePop->m_iNumSpecies || iSpecies < 0) {
     //invalid species - throw an error
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::CalcSeedlingHeight");
     sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies);
     throw(stcErr);
   }
   fHeight = 0.1 +
               30*(1.0-exp((double)
               (-mp_oTreePop->mp_fSlopeHeightDiam10[iSpecies]*fDiam10)));
   return fHeight;
 }
 catch(modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clAllometry::CalcSeedlingHeight");
   throw(stcErr);
 }
}
//--------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
CalcSeedlingDiam10()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////*/
float clAllometry::CalcSeedlingDiam10( float fHeight, int iSpecies ) {
 try {
   float fDiam10 = 0;
   if (iSpecies >= mp_oTreePop->m_iNumSpecies || iSpecies < 0) {
     //invalid species - throw an error
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::CalcSeedlingDiam10");
     sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies);
     throw(stcErr);
   }
   //If the height is greater than or equal to 30.1, throw bad data error
   if (fHeight >= 30.1) {
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::CalcAdultDbh");
     sprintf(stcErr.cMoreInfo, "%s%s", "Cannot set tree height greater than ",
         "maximum sapling height for this species.");
     throw(stcErr);
   }
   fDiam10 = (log(1-((fHeight-0.1)/30)))/
                         -mp_oTreePop->mp_fSlopeHeightDiam10[iSpecies];
   return fDiam10;
 }
 catch(modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clAllometry::CalcSeedlingDiam10");
   throw(stcErr);
 }
}
//--------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
ConvertDiam10ToDbh()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////*/
float clAllometry::ConvertDiam10ToDbh( float fDiam10, int iSpecies ) {
 try {
   float fDbh = 0;
   if (iSpecies >= mp_oTreePop->m_iNumSpecies || iSpecies < 0) {
     //invalid species - throw an error
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::ConvertDiam10ToDbh");
     sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies);
     throw(stcErr);
   }
   fDbh = fDiam10 * mp_oTreePop->mp_fDbhToDiam10Ratio[iSpecies];
   return fDbh;
 }
 catch(modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clAllometry::ConvertDiam10ToDbh");
   throw(stcErr);
 }
}
//--------------------------------------------------------------------------

/*///////////////////////////////////////////////////////////////////////////
ConvertDbhToDiam10()
// Edit history:
// -------------
// April 28, 2004 - Submitted in beta version (LEM)

///////////////////////////////////////////////////////////////////////////*/
float clAllometry::ConvertDbhToDiam10( float fDbh, int iSpecies ) {
 try {
   float fDiam10 = 0;
   if (iSpecies >= mp_oTreePop->m_iNumSpecies || iSpecies < 0) {
     //invalid species - throw an error
     modelErr stcErr;
     stcErr.iErrorCode = BAD_DATA;
     strcpy(stcErr.cFunction, "clAllometry::ConvertDbhToDiam10");
     sprintf(stcErr.cMoreInfo, "%s%d", "Unrecognized species ", iSpecies);
     throw(stcErr);
   }
   fDiam10 = fDbh / mp_oTreePop->mp_fDbhToDiam10Ratio[iSpecies];
   return fDiam10;
 }
 catch(modelErr &err) {throw(err);}
 catch (modelMsg &msg) {throw(msg);} //non-fatal error
 catch (...) {
   modelErr stcErr;
   stcErr.iErrorCode = UNKNOWN;
   strcpy(stcErr.cFunction, "clAllometry::ConvertDbhToDiam10");
   throw(stcErr);
 }
}
//--------------------------------------------------------------------------

